import logging
from logging import getLogger
from typing import List, Optional
from pathlib import Path

from .util import create_temporary_docker_volume
from .volume import copy_input_to, copy_output_from
from .preprocessing import apply_super_resolution
from .ocr import run_tesseract, run_google_vision_ai, combine_results

import click
import docker


LOGGER = getLogger(__name__)


@click.command()
@click.argument('input-images',
                type=click.Path(exists=True, dir_okay=False, file_okay=True),
                nargs=-1,
                required=True)
@click.argument('output-dir',
                type=click.Path(exists=True, dir_okay=True, file_okay=False),
                required=True)
@click.option('--google-vision-ai',
              help='Use Google Vision AI in addition to Tesseract for OCR (recommended, requires paid account)',
              default=False,
              is_flag=True,
              required=False)
@click.option('--google-api-key',
              help='A file with the API key for the Google Vision AI',
              type=click.Path(exists=True, dir_okay=False, file_okay=True),
              default=None,
              required=False)
@click.option('--super-resolution',
              help='Use image super-resolution before the OCR (recommended, requires GPU)',
              default=False,
              is_flag=True,
              required=False)
def main(input_images: List[str], output_dir: str, google_vision_ai: bool,
         google_api_key: Optional[str], super_resolution: bool) -> None:

    if google_vision_ai and google_api_key is None:
        raise ValueError('Cannot use Google Vision AI without an API key.')
    client = docker.from_env()

    def check_docker_image(name: str) -> None:
        if not client.images.list(name):
            raise ValueError(f'Docker image {name} not found. Install it with make build.')

    check_docker_image('ahisto/empty')
    check_docker_image('ahisto/ocr-eval')
    check_docker_image('ahisto/ocr-fileformat')
    check_docker_image('ahisto/tesseract')
    check_docker_image('ahisto/waifu2x')

    input_images = sorted(map(Path, input_images))
    output_dir = Path(output_dir)
    if google_api_key is not None:
        google_api_key = Path(google_api_key)

    logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s')

    run_ocr(input_images, output_dir, google_vision_ai, google_api_key, super_resolution)


def run_ocr(input_images: List[Path], output_dir: Path, google_vision_ai: bool,
            google_api_key: Optional[Path], super_resolution: bool) -> None:
    client = docker.from_env()
    with create_temporary_docker_volume(client) as input_volume, \
            create_temporary_docker_volume(client) as postprocessing_volume, \
            create_temporary_docker_volume(client) as tesseract_volume, \
            create_temporary_docker_volume(client) as google_vision_ai_volume, \
            create_temporary_docker_volume(client) as output_volume:

        copy_input_to(client, input_volume, input_images, google_api_key)

        if super_resolution:
            apply_super_resolution(client, input_volume, postprocessing_volume)
        else:
            LOGGER.info('Skipping image super-resolution')
            postprocessing_volume = input_volume

        run_tesseract(client, postprocessing_volume, tesseract_volume)

        if google_vision_ai:
            LOGGER.info('Skipping Google Vision AI')
            run_google_vision_ai(client, postprocessing_volume, google_vision_ai_volume)
            combine_results(client, tesseract_volume, google_vision_ai_volume, output_volume)
        else:
            output_volume = tesseract_volume

        copy_output_from(client, output_volume, input_images, output_dir)
