from logging import getLogger
from pathlib import Path

from .config import CONFIG as _CONFIG
from .util import (
    create_temporary_docker_volume,
    run_docker_container,
    extract_text_file_from_container,
    add_text_file_to_container,
)


LOGGER = getLogger(__name__)
CONFIG = _CONFIG['ocr']


def run_tesseract(client, preprocessing_volume, second_pass_volume) -> None:

    with create_temporary_docker_volume(client) as first_pass_volume:
        run_tesseract_first_pass(client, preprocessing_volume, first_pass_volume)
        run_tesseract_second_pass(client, preprocessing_volume, first_pass_volume, second_pass_volume)

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        second_pass_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }
    with create_temporary_docker_container(client, 'ahisto/empty', command='cmd',
                                           volumes=volumes) as container:
        text = extract_text_file_from_container(container, '/input/list.txt')
        add_text_file_to_container(container, '/output/list.txt', text)


def run_tesseract_first_pass(client, preprocessing_volume, first_pass_volume) -> None:
    LOGGER.info('Running the first pass of Tesseract with all languages')

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        first_pass_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '25%', '--',
        f'nice -n {CONFIG.getint("niceness")} tesseract {{}} /output/{{/.}} -l {CONFIG["languages"]} '
        '--oem 1 --psm 3 --tessdata-dir /tessdata hocr', '::::', '/input/list.txt',
    ]
    run_docker_container(client, 'ahisto/tesseract', command=command, volumes=volumes)

    command = [
        'python', '-m', 'scripts.extract_detected_languages', '/input/list.txt', '/output',
        CONFIG["languages"], '/output/detected-languages.txt', '00',
    ]
    run_docker_container(client, 'ahisto/ocr-eval', command=command, volumes=volumes)


def run_tesseract_second_pass(client, preprocessing_volume, first_pass_volume,
                              second_pass_volume) -> None:
    LOGGER.info('Running the second pass of Tesseract with selected languages')

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        first_pass_volume.name: {
            'bind': '/first-pass',
            'mode': 'ro',
        },
        second_pass_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '25%', '-N', '2', '--',
        'if [ {2} = None ]; then echo | tee /output/{1/.}.{hocr,txt,tsv,box,page}; else '
        f'nice -n {CONFIG.getint("niceness")} tesseract {{1}} /output/{{1/.}} -l {{2}} '
        '--oem 1 --psm 3 --tessdata-dir /tessdata hocr txt tsv makebox; fi', '::::',
        '/first-pass/detected-languages.txt'
    ]
    run_docker_container(client, 'ahisto/tesseract', command=command, volumes=volumes)

    del volumes[first_pass_volume.name]

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '100%', '--',
        f'! grep -q . /output/{{/.}}.hocr || nice -n {CONFIG.getint("niceness")} ocr-transform '
        'hocr page /output/{/.}.hocr /output/{/.}.page', '::::', '/input/list.txt',
    ]
    run_docker_container(client, 'ahisto/ocr-fileformat', command=command, volumes=volumes)


def run_google_vision_ai(client, postprocessing_volume, google_vision_ai_volume) -> None:
    LOGGER.info('Running Google Vision AI')

    volumes = {
        preprocessing_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        google_vision_ai_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'parallel', '--halt', 'now,fail=1', '--jobs', '100%', '--',
        f'nice -n {CONFIG.getint("niceness")} scripts/google-vision-annotate.sh '
        '/input/google-api-key.txt {} /output/{/.}.json /output/{/.}.error', '::::',
        '/input/list.txt',
    ]
    run_docker_container(client, 'ahisto/ocr-eval', command=command, volumes=volumes)


def combine_results(client, tesseract_volume, google_vision_ai_volume, output_volume):
    LOGGER.info('Combining Tesseract with Google Vision AI')

    volumes = {
        tesseract_volume.name: {
            'bind': '/input',
            'mode': 'ro',
        },
        google_vision_ai_volume.name: {
            'bind': '/google-vision-ai',
            'mode': 'ro',
        },
        output_volume.name: {
            'bind': '/output',
            'mode': 'rw',
        },
    }

    command = [
        'python', '-c', r"""
        from pathlib import Path
        import shutil

        from scripts.common import is_multicolumn


        with open('/input/list.txt', 'rt') as rf, open('/output/detected-layout.txt', 'wt') as wf:
            for line in rf:
                filename = Path(line.rstrip('\r\n'))
                input_tesseract_filename = Path('/input') / f'{filename.stem}.txt'
                input_google_filename = Path('/google-vision-ai') / f'{filename.stem}.txt'
                output_filename = Path('/output') / f'{filename.stem}.txt'
                decision_filename = Path('/output') / f'{filename.stem}.ground-truth'

                with input_tesseract_filename.with_suffix('.hocr').open('rt') as f:
                    hocr_content = f.read()
                if not hocr_content.strip():
                    with output_filename.open('wt') as f:
                        print(file=f)
                    continue

                if not input_google_filename.exists():
                    shutil.copy(input_tesseract_filename, output_filename)

                if is_multicolumn(input_tesseract_filename.with_suffix('.hocr')):
                    shutil.copy(input_tesseract_filename, output_filename)
                    with decision_filename.open('wt') as f:
                        print('hocr', file=f)
                else:
                    shutil.copy(input_google_filename, output_filename)
                    with decision_filename.open('wt') as f:
                        print('json', file=f)
        """,
    ]
    run_docker_container(client, 'ahisto/ocr-eval', command=command, volumes=volumes)
