from contextlib import contextmanager
from logging import getLogger
import tarfile
from tarfile import TarFile, TarInfo
from tempfile import TemporaryFile
from time import time
from typing import Iterator
from contextlib import closing
from io import BytesIO
import os
import sys

from docker.types import DeviceRequest
from docker.errors import ContainerError


LOGGER = getLogger(__name__)


@contextmanager
def create_temporary_docker_volume(client, *args, **kwargs) -> Iterator:
    volume = client.volumes.create(*args, **kwargs)
    LOGGER.debug(f'Created temporary Docker volume {volume.name}')
    try:
        yield volume
    finally:
        volume.remove(force=True)
        LOGGER.debug(f'Deleted temporary Docker volume {volume.name}')


@contextmanager
def create_temporary_docker_container(client, *args, gpu: bool = False, **kwargs) -> Iterator:
    if gpu:
        kwargs = {**kwargs, **{'device_requests': [DeviceRequest(count=-1, capabilities=[['gpu']])]}}
    container = client.containers.create(*args, **kwargs)
    LOGGER.debug(f'Created temporary Docker container {container.short_id}')
    try:
        yield container
    finally:
        container.remove(force=True)
        LOGGER.debug(f'Deleted temporary Docker container {container.short_id}')


def run_docker_container(client, *args, gpu: bool = False, quiet: bool = True, **kwargs) -> None:
    kwargs = {**{'remove': True, 'stdout': True, 'stderr': True}, **kwargs}
    if gpu:
        kwargs = {**kwargs, **{'device_requests': [DeviceRequest(count=-1, capabilities=[['gpu']])]}}
    try:
        output = client.containers.run(*args, **kwargs)
        if not quiet:
            with os.fdopen(sys.stdout.fileno(), 'wb', closefd=False) as stdout:
                stdout.write(output)
    except ContainerError as e:
        e.container.remove(force=True)
        raise e


def add_text_file_to_tarfile(filename: str, text: str, tar: TarFile) -> None:
    with closing(BytesIO(text.encode('utf-8'))) as f:
        tarinfo = TarInfo(filename)
        tarinfo.size = len(f.getvalue())
        tarinfo.mtime = time()
        tar.addfile(tarinfo, fileobj=f)


def add_text_file_to_container(container, filename: str, text: str) -> None:
    with TemporaryFile('w+b') as tf:
        with tarfile.open(fileobj=tf, mode='w') as tar:
            add_text_file_to_tarfile(filename, text, tar)
        tf.seek(0)
        container.put_archive(path='/', data=tf)


def extract_text_file_from_container(container, filename: str) -> str:
    with TemporaryFile('w+b') as tf:
        bits, _ = container.get_archive(filename)
        for chunk in bits:
            tf.write(chunk)
        tf.seek(0)
        text = tf.read().decode('utf-8')
    return text
